using KPGBase.Excercises;

namespace KPGBase
{
    public partial class Form1 : Form
    {
        Excercise01 ex01;
        Excercise02 ex02;
        Excercise03 ex03;
        Excercise04 ex04;
        Excercise05 ex05;

        Random r;
        System.Windows.Forms.Timer timer;

        bool leftClicked;

        public Form1()
        {
            InitializeComponent();
            FormClosed += StopTimer;
            r = new Random();

            canvasBox.Image = new Bitmap(canvasBox.Width, canvasBox.Height);
            ex01 = new Excercise01((Bitmap)canvasBox.Image);
            ex02 = new Excercise02((Bitmap)canvasBox.Image);
            ex03 = new Excercise03((Bitmap)canvasBox.Image);
            ex04 = new Excercise04((Bitmap)canvasBox.Image);
            ex05 = new Excercise05((Bitmap)canvasBox.Image);

            canvasBox.MouseDown += OnCanvasClick;
            canvasBox.MouseMove += OnCanvasMove;
            canvasBox.MouseUp += OnCanvasUp;
        }

        /// <summary>
        /// Stop moving bezier control point
        /// </summary>
        private void OnCanvasUp(object? sender, MouseEventArgs e)
        {
            leftClicked = false;
        }

        /// <summary>
        /// If active curves tab and left mouse button clicked, the user is trying to move a bezier control point
        /// </summary>
        private void OnCanvasMove(object? sender, MouseEventArgs e)
        {
            // if active curves tab
            if (tabControl1.SelectedIndex != 4)
                return;
            Point newLoc = e.Location;
            // if left clicked move point to new location
            if (leftClicked)
                ex05.MoveSelectedPoint(newLoc);

            canvasBox.Invalidate();
        }

        /// <summary>
        /// If active curves tab and left button clicked the user is trying to add a new bezier control point
        /// </summary>
        private void OnCanvasClick(object? sender, MouseEventArgs e)
        {
            // if active curves tab
            if (tabControl1.SelectedIndex != 4)
                return;

            if (e.Button == MouseButtons.Left)
            {
                // add point to control points
                Point startPoint = e.Location;
                leftClicked = true;
                ex05.SelectPoint(startPoint);
            }
            
            canvasBox.Invalidate();
        }

        private void StopTimer(object? sender, FormClosedEventArgs e)
        {
            if (isRunning)
                timer.Stop();
        }

        private void sandboxBT_Click(object sender, EventArgs e)
        {
            ex01.DrawingSandbox();
            canvasBox.Invalidate();
        }

        bool isRunning = false;
        private void animBT_Click(object sender, EventArgs e)
        {
            if (isRunning)
            {
                timer.Stop();
                isRunning = false;
            }
            else
            {
                timer = new System.Windows.Forms.Timer();
                timer.Interval = 1;
                timer.Tick += new EventHandler(NextStepSquare);
                timer.Enabled = true;
                timer.Start();
                isRunning = true;
            }
        }

        private void NextStepSquare(object? sender, EventArgs e)
        {
            ex01.NextStepSquare();
            canvasBox.Invalidate();
        }

        private void mazeBT_Click(object sender, EventArgs e)
        {
            // generate random number of columns and rows
            // min size of maze is 2x2
            int m = r.Next(30) + 2;
            int n = r.Next(30) + 2;
            // find out maximum size of the cell so the maze still fits the window
            // min padding equals the size of a cells on each side of the maze
            int cellsizeCol = canvasBox.Image.Width / (n + 2);
            int cellsizRow = canvasBox.Image.Height / (m + 2);
            int cellsize = Math.Min(cellsizRow, cellsizeCol);
            ex02.GenerateMaze(m, n, cellsize);
            canvasBox.Invalidate();
        }

        bool lifeInit = false;
        private void lifeBT_Click(object sender, EventArgs e)
        {
            int m = 20;
            int n = 20;
            int cellsizeCol = canvasBox.Image.Width / (n + 2);
            int cellsizRow = canvasBox.Image.Height / (m + 2);
            int cellsize = Math.Min(cellsizRow, cellsizeCol);
            ex03.GenerateLife(m, n, cellsize);
            canvasBox.Invalidate();
            lifeInit = true;
        }

        private void runLifeBT_Click(object sender, EventArgs e)
        {
            if (!lifeInit)
                return;

            if (isRunning)
            {
                timer.Stop();
                isRunning = false;
            }
            else
            {
                timer = new System.Windows.Forms.Timer();
                timer.Interval = 100;
                timer.Tick += new EventHandler(NextStepLife);
                timer.Enabled = true;
                timer.Start();
                isRunning = true;
            }
        }

        private void NextStepLife(object? sender, EventArgs e)
        {
            ex03.NextLifeConfig();
            canvasBox.Invalidate();
        }

        private void simpleTileBT_Click(object sender, EventArgs e)
        {
            ex04.SimpleTiling();
            canvasBox.Invalidate();
        }

        private void triangleTileBT_Click(object sender, EventArgs e)
        {
            ex04.TriangleTiling();
            canvasBox.Invalidate();
        }

        private void spiralTileBT_Click(object sender, EventArgs e)
        {
            ex04.SpiralTiling();
            canvasBox.Invalidate();
        }

        private void bezierBT_Click(object sender, EventArgs e)
        {
            ex05.DrawBezierCurve();
            canvasBox.Invalidate();
        }

        private void clearPts_Click(object sender, EventArgs e)
        {
            ex05.ClearPoints();
            canvasBox.Invalidate();
        }
    }
}